# -*- coding: utf-8 -*-
"""Traitement de fichiers de notes.csv
Chaque ligne présente la structure Nom Prenom,note1,note2, ...,note6\n
Le caractère de fin de ligne est \n (New Line ou Line Feed sous Unix
 ou \r\n  sous Windows (Carriage Return Line Feed)"""
import matplotlib.pyplot as plt

def moyenne(liste,coefs):
    """retourne la moyenne d'une liste  de flottants """
    #syntaxe idiomatique
    #return  sum([note*c  for note,c in zip(liste,coefs)])/sum(coefs) 
    return round(sum([liste[i]*coefs[i] for i in range(len(liste))])/sum(coefs),2)
    
def fichier_moyennes(fichier='notesBCPST.csv',coefs=[i+1 for i in range(6)]):
    """ouvre le fichier 1 en lecture, ouvre le fichier2 en écriture.
    On recopie dans notes2.csv la ligne lue dans notesBCPST.csv et on rajoute en fin de ligne 
    (en enlevant d'abord le caractère '\n') le caractère '|' puis la moyenne de l'élève arrondie 
    à 2 chiffres après la virgule puis le caractère de fin de ligne '\n' """
    
    f = open(fichier,'r')
    g = open(fichier.rstrip('.csv')+'-moyennes.csv','w')
    #liste des moyennes d'élèves
    lmoy= []
    for ligne in f:
        #on enlève le caractère non imprimable de fin de ligne
        #puis on récupère les valeurs des différents champs séparés par des virgules
        #dans une liste
        ligne = ligne.rstrip('\n')
        liste = ligne.split(',')
        #on récupère les notes dans une liste de flottants
        notes = [float(e) for e in liste[1:]]
        #moyenne de l'élève
        m = moyenne(notes,coefs)
        #ajout de la moyenne de l'élève à la liste des moyennes
        lmoy.append(m)
        #on recopie la ligne lue + la moyenne de l'élève
        #g.write(ligne+',%.2f'%m)
        g.write(ligne+'|'+str(round(m,2))+'\n')
    #on rajoute une ligne à la fin avec la moyenne générale
    g.write('Moyenne générale : %.2f'%moyenne(lmoy,[1]*len(lmoy)))
    g.close()
    f.close()
    #on retourne la liste des moyennes pour une exploitation graphique par exemple
    return lmoy

def bilan_graphique(fichier,coefs=[i+1 for i in range(6)]):
    lmoy = fichier_moyennes(fichier,coefs)
    #création de l'histogramme des notes
    plt.subplot(121)
    plt.title('Histogramme des moyennes')
    plt.hist(lmoy,range=(0,20),bins=20,color='blue')
    plt.subplot(122)
    #création du diagramme en boite
    plt.title('Diagramme en boite des moyennes')
    plt.ylim(0,20)
    plt.boxplot(lmoy)
    plt.show()
    plt.savefig('HistoBilanBCPST.png')

def lecture_dico(fichier1='notesBCPST.csv'):
    """ouvre le fichier de notes fichier 1
    et retourne  un dictionnaire dico
    dont les clefs sont les chaines 'NOM Prenom' et les 
    valeurs sont les listes de notes des élèves correspondants.
    Fonction dépréciée"""
    f = open(fichier1,'r')
    #création d'un dictionnaire vide
    dico = dict()
    for ligne in f:
        #on enlève le caractère non imprimable de fin de ligne
        #puis on récupère les valeurs des différents champs séparés par des virgules
        #dans une liste
        ligne = ligne.rstrip('\n')
        liste = ligne.split(',')
        dico[liste[0]] = [int(note) for note in liste[1:]]
    f.close()
    return dico

def lecture(fichier='notesBCPST.csv',coefs=[i+1 for i in range(6)]):
    """ouvre le fichier de notes fichier 1
    et retourne  une liste de sous-listes représentant 
    toutes les lignes du fichier et la liste des moyennes générales.
    Chaque sous-liste représente une ligne.
    Une   sous-liste contient
    le nom de l'élève, puis ses notes, puis sa moyenne 
    en fonction de la liste de coefficients coefs.
    Tous les objets des sous-listes sont des chaines de caractères
    car celles-ci sont destinées à être écrites dans un fichier texte.
    """
    f = open(fichier,'r')
    liste = []
    #liste des moyennes d'élèves
    lmoy = []
    #liste des listes de notes aux devoirs
    notesdevoirs = [[] for i in range(len(coefs))]
    for ligne in f:
        #on enlève le caractère non imprimable de fin de ligne
        #puis on récupère les valeurs des différents champs séparés par des virgules
        #dans une liste
        ligne = ligne.rstrip('\n') #ou ligne = ligne.rstrip('\r\n') sous Windows
        l = ligne.split(',')
        #on récupère les notes dans une liste de flottants
        notes = [float(e) for e in l[1:]]
        #ajout des notes aux listes de notes par devoir
        for i in range(len(notes)):
            notesdevoirs[i].append(notes[i])
        #calcul de la moyenne de l'élève
        m = moyenne(notes,coefs)
        #ajout de la moyenne de l'élève à la liste des moyennes (qui est une liste de flottants)
        lmoy.append(m)
        #ajout de la moyenne à la sous-liste représentant l'élève
        l.append(str(m))
        #ajout de la sous-liste à la liste
        liste.append(l)
    #ajout de la liste moyennes eleves à la listes des notes aux devoirs
    notesdevoirs.append(lmoy)
    #tri sur place dans l'ordre décroissant de la liste des moyennes 
    #la liste moyennes est une liste de flottants
    lmoy.sort(reverse=True)
    #ajout du rang de l'élève à la fin de la sous-liste le représentant
    for l in liste:
        l.append(str(lmoy.index(float(l[-1]))+1))
    #ajout d'une sous(liste qui représentera la ligne des moyennes par DS + la moyenne générale
    liste.append(['MOYENNES'])
    #calcul des moyennes par DS et de la moyenne générale
    for devoir in notesdevoirs:
        m = moyenne(devoir,[1]*len(devoir))
        #ajout de la moyenne calculée à la dernière sous-liste représentant les moyennes
        liste[-1].append(str(m))        
    f.close()
    return liste,lmoy

def fichier_bilan(fichier='notesBCPST.csv',coefs=[i+1 for i in range(6)]):
    """A partir du fichier de notes fichier, génère un fichier texte bilan du trimestre
     avec les moyennes et classements et un histogramme"""
    #on lit notre fichier et on constitue notre liste de lignes
    #avec toutes les infos nécessaires (moyennes, classements)
    listelignes,lmoy = lecture(fichier,coefs)
    print(listelignes)
    #on ouvre notre fichier texte bilan en mode écriture
    g = open('bilan_notesBCPST.csv','w')
    #on parcourt notre liste de sous-listes représentant les lignes
    for l in listelignes:
        #on parcourt les éléments de chaque sous-liste
        i = 0
        while i<len(l)-1:
            g.write(l[i]+',')
            i += 1
        #écriture du dernier élément  suivi du caractère de fin de ligne
        #g.write(l[i]+'\r'\n') sous Windows
        g.write(l[i]+'\n')      
    g.close()
    #création de l'histogramme des notes
    plt.subplot(121)
    plt.title('Histogramme des moyennes')
    plt.hist(lmoy,range=(0,20),bins=20,color='blue')
    plt.subplot(122)
    plt.title('Diagramme en boite des moyennes')
    plt.ylim(0,20)
    plt.boxplot(lmoy)
    plt.savefig('HistoBilanBCPST.png')
    plt.show()
        
    

#listelignes  = lecture()
#print(listelignes)
fichier_bilan()
#bilan_graphique('notesBCPST.csv')
        
        
    

        
    
    
